
class AccountUserTestingMixin(object):
    """
    Common setup required for testing accounts
    """

    def create_users(self):
        from django.contrib.auth.models import User
        self.user = User.objects.create_user(username="user", email="", password="pass")
        self.seconduser = User.objects.create_user(username="lucy", email="", password="pass")
        self.thirduser = User.objects.create_user(username="bob", email="", password="pass")
        self.fourthuser = User.objects.create_user(username="sue", email="", password="pass")
        return self.user, self.seconduser, self.thirduser, self.fourthuser

    def create_accounts(self):
        from accounts.models import Account
        accountone = Account.objects.create(name="one")
        accounttwo= Account.objects.create(name="two")
        return accountone, accounttwo

    def get_fixtures(self):
        from accounts.models import AccountUser, AccountOwner
        users = self.create_users()
        self.account1, self.account2 = self.create_accounts()
        self.accountuser_1 = AccountUser.objects.create(
                user=users[0],
                account=self.account1)
        self.accountuser_2 = AccountUser.objects.create(
                user=users[1],
                account=self.account1)
        self.accountuser_3 = AccountUser.objects.create(
                user=users[2],
                account=self.account2)
        self.owner = AccountOwner.objects.create(
                account=self.account1,
                account_user=self.accountuser_1)




from django.test import TestCase
from django.core.urlresolvers import reverse

from testing_utils import AccountUserTestingMixin


class BaseAccountUnauthViewTest(AccountUserTestingMixin, TestCase):
    """
    Ensure all views require logged in users
    """
    def setUp(self):
        self.get_fixtures()

    def test_account_list(self):
        response = self.client.get(reverse("base_account_list"))
        self.assertEqual(response.status_code, 302)

    def test_account_add(self):
        response = self.client.get(reverse("base_account_add"))
        self.assertEqual(response.status_code, 302)

    def test_account_detail(self):
        response = self.client.get(reverse("base_account_detail",
            kwargs={"account_pk": self.account1.pk}))
        self.assertEqual(response.status_code, 302)

    def test_account_update(self):
        response = self.client.get(reverse("base_account_edit",
            kwargs={"account_pk": self.account1.pk}))
        self.assertEqual(response.status_code, 302)

    def test_account_delete(self):
        response = self.client.get(reverse("base_account_delete",
            kwargs={"account_pk": self.account1.pk}))
        self.assertEqual(response.status_code, 302)

    def test_accountuser_list(self):
        response = self.client.get(reverse("base_accountuser_list",
            kwargs={"account_pk": self.account1.pk}))
        self.assertEqual(response.status_code, 302)

    def test_accountuser_add(self):
        response = self.client.get(reverse("base_accountuser_add",
            kwargs={"account_pk": self.account1.pk}))
        self.assertEqual(response.status_code, 302)

    def test_accountuser_detail(self):
        response = self.client.get(reverse("base_accountuser_detail", kwargs={
            "account_pk": self.account1.pk,
            "accountuser_pk": self.accountuser_1.pk}))
        self.assertEqual(response.status_code, 302)

    def test_accountuser_update(self):
        response = self.client.get(reverse("base_accountuser_edit", kwargs={
            "account_pk": self.account1.pk,
            "accountuser_pk": self.accountuser_1.pk}))
        self.assertEqual(response.status_code, 302)

    def test_accountuser_delete(self):
        response = self.client.get(reverse("base_accountuser_delete", kwargs={
            "account_pk": self.account1.pk,
            "accountuser_pk": self.accountuser_1.pk}))
        self.assertEqual(response.status_code, 302)


class BaseAccountViewTest(AccountUserTestingMixin, TestCase):
    """
    Verify GET requests on all views. This does not cover updates via POST
    requests.
    """
    def setUp(self):
        self.get_fixtures()
        self.client.login(username="user", password="pass")

    def test_account_list(self):
        """Ensure that the accounts can be listed"""
        response = self.client.get(reverse("base_account_list"))
        self.assertEqual(response.status_code, 200)

    def test_account_add(self):
        """Ensure that the account creation view is accessible"""
        response = self.client.get(reverse("base_account_add"))
        self.assertEqual(response.status_code, 200)

    def test_account_detail(self):
        """Ensure that the account detail view is accessible"""
        response = self.client.get(reverse("base_account_detail",
            kwargs={"account_pk": self.account1.pk}))
        self.assertEqual(response.status_code, 200)

    def test_account_update(self):
        """Ensure that the account update view is accessible"""
        response = self.client.get(reverse("base_account_edit",
            kwargs={"account_pk": self.account1.pk}))
        self.assertEqual(response.status_code, 200)

    def test_account_delete(self):
        """Ensure that the account deletion view is accessible"""
        response = self.client.get(reverse("base_account_delete",
            kwargs={"account_pk": self.account1.pk}))
        self.assertEqual(response.status_code, 200)

    def test_accountuser_list(self):
        """Ensure that the account users can be listed"""
        response = self.client.get(reverse("base_accountuser_list",
            kwargs={"account_pk": self.account1.pk}))
        self.assertEqual(response.status_code, 200)

    def test_accountuser_add(self):
        """Ensure that the account user creation view is accessible"""
        response = self.client.get(reverse("base_accountuser_add",
            kwargs={"account_pk": self.account1.pk}))
        self.assertEqual(response.status_code, 200)

    def test_accountuser_detail(self):
        """Ensure that the account user detail view is accessible"""
        response = self.client.get(reverse("base_accountuser_detail", kwargs={
            "account_pk": self.account1.pk,
            "accountuser_pk": self.accountuser_1.pk}))
        self.assertEqual(response.status_code, 200)

    def test_accountuser_update(self):
        """Ensure that the account user update view is accessible"""
        response = self.client.get(reverse("base_accountuser_edit", kwargs={
            "account_pk": self.account1.pk,
            "accountuser_pk": self.accountuser_1.pk}))
        self.assertEqual(response.status_code, 200)

    def test_accountuser_delete(self):
        """Ensure that the account user delete view is accessible"""
        response = self.client.get(reverse("base_accountuser_delete", kwargs={
            "account_pk": self.account1.pk,
            "accountuser_pk": self.accountuser_1.pk}))
        self.assertEqual(response.status_code, 200)


class BaseCreationViewsTest(AccountUserTestingMixin, TestCase):
    """
    Test the views for creating accounts and account users
    """
    def setUp(self):
        self.get_fixtures()
        self.client.login(username="user", password="pass")

    def test_simple_valid_account(self):
        """Ensure that basic account creation view works"""
        from accounts.models import Account
        start_count = Account.objects.all().count()
        response = self.client.post(reverse("base_account_add"), data={
            "account_name": "Da Bears",
            "username": "fridge",
            "first_name": "William",
            "last_name": "Perry",
            "email": "72@bears.com",})
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Account.objects.all().count(), start_count + 1)

    def test_simple_valid_accountuser(self):
        """Ensure that basic account user creation view works"""
        from accounts.models import AccountUser
        start_count = AccountUser.objects.all().count()
        post_data = {
            "username": "thedude",
            "first_name": "Jeffrey",
            "last_name": "Lebowski",
            "email": "jeff@bowlorama.com",
            "password": "roll",
        }
        response = self.client.post(reverse("base_accountuser_add",
            kwargs={"account_pk": 1}), data=post_data)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(AccountUser.objects.all().count(), start_count + 1)


class BaseDeletionViewsTest(AccountUserTestingMixin, TestCase):
    """
    Test the views for updating accounts and account users
    """
    def setUp(self):
        self.get_fixtures()
        self.client.login(username="user", password="pass")

    def test_simple_account_deletion(self):
        """Ensure that the account delete view deletes"""
        from accounts.models import Account
        start_count = Account.objects.all().count()
        response = self.client.post(reverse("base_account_delete",
            kwargs={"account_pk": 2}),data={"pk": 2})
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Account.objects.all().count(), start_count - 1)


class BaseUpdateDeletionViewsTest(AccountUserTestingMixin, TestCase):
    """
    Test the views for deleting accounts and account users
    """
    def setUp(self):
        self.get_fixtures()
        self.client.login(username="user", password="pass")

    def test_simple_account_update(self):
        """Ensure that the account update view updates"""
        response = self.client.post(reverse("base_account_edit",
            kwargs={"account_pk": 2}),data={"name": "Updated"})
        self.assertEqual(response.status_code, 302)

    def test_simple_accountuser_update(self):
        """Ensure that the account updates view updates"""
        response = self.client.post(reverse("base_accountuser_edit",
            kwargs={"account_pk": 2, "accountuser_pk": 1}),
            data={"username": "newusername"})
        self.assertEqual(response.status_code, 302)




from django.conf.urls.defaults import url, patterns, include

from django.contrib import admin

urlpatterns = patterns('',
    url(r'^admin/', include(admin.site.urls)),
    url(r'^accounts/', include('accounts.urls')),
    url(r'^accounts_base/', include('accounts.urls.base')),
)




from django.test import TestCase

from testing_utils import AccountUserTestingMixin


class AccountModelsTest(TestCase, AccountUserTestingMixin):
    """
    Verify that the objects are created as intended, and that the managers
    return querysets as intended.
    """
    def setUp(self):
        from django.contrib.auth.models import User
        self.user = User.objects.create_user(
            username="tester",
            email="test@example.com"
        )

    def test_account_model(self):
        """Ensure that creating account w/ specific attributes doesn't fail"""
        from accounts.models import Account
        account = Account.objects.create(name="My test account")

    def test_create_account(self):
        """create_account method should create an account and an owner user"""
        from accounts.models import Account
        from accounts.utils import create_account
        new_account = create_account(
                name="My test account",
                owner=self.user)
        self.assertTrue(isinstance(new_account, Account))

        self.assertRaises(TypeError, 'create_user()',
                create_account, name="No owner")

    def test_active_manager(self):
        """Only return active accounts"""
        from accounts.models import Account
        account1 = Account.objects.create(name="My test account")
        account2 = Account.objects.create(name="My test account")
        account3 = Account.objects.create(name="My test account",
                is_active=False)
        self.assertEqual(2, len(Account.objects.active()))


class UserAccountRelationships(TestCase, AccountUserTestingMixin):
    """
    Test the expected relationship constraints between the Users, Accounts and
    AccountUsers. Should be able to get a queryset of all Accounts based on a
    single User object.
    """

    def setUp(self):
        from django.contrib.auth.models import User
        self.user = User.objects.create_user("user", "none",
                "user@example.com")
        self.seconduser = User.objects.create_user("jerk", "none",
        "second@example.com")

    def test_multiple_account_ownership(self):
        """Ensure that a User can be a member of and own multiple accounts"""
        from accounts.models import AccountUser
        from accounts.utils import create_account
        account1 = create_account(name="one", owner=self.user)
        account2 = create_account(name="two", owner=self.user)
        account3 = create_account(name="three", owner=self.seconduser)
        additional_account = AccountUser.objects.create(account=account3,
                user=self.user)


class AccountOwnership(TestCase, AccountUserTestingMixin):
    """
    Test the constraints on account ownership, ensure that a user can own
    multiple accounts, and make sure that ownership can be transferred, and
    that an account can't become an orphan. Make sure that only account users
    for the given account can be owners.
    """

    def setUp(self):
        from django.contrib.auth.models import User
        self.user = User.objects.create_user("user", "none",
                "user@example.com")
        self.seconduser = User.objects.create_user("lucy", "none",
                "second@example.com")
        self.thirduser = User.objects.create_user("bob", "none",
                "third@example.com")

    def test_manual_creation(self):
        """Just ensure no errors arise"""
        from accounts.models import Account, AccountUser, AccountOwner
        second_account = Account.objects.create(name="second")
        second_account_user = AccountUser.objects.create(
                account=second_account,
                user=self.seconduser,
        )
        second_account_owner = AccountOwner.objects.create(
                account=second_account,
                account_user=second_account_user
        )

    def test_one_account_owner(self):
        """Ensure only one account owner exists"""
        from django.db import IntegrityError
        from accounts.models import Account, AccountUser, AccountOwner
        from accounts.utils import create_account
        new_account = create_account(
                name="My test account",
                owner=self.user)
        # Account object creation is implied by owernship creation
        self.assertEqual(new_account.owner.account_user.user, self.user)

        new_account_user = AccountUser.objects.create(
                account=new_account,
                user=self.seconduser,
        )
        self.assertRaises(IntegrityError,
                AccountOwner.objects.create, **{
                    'account': new_account,
                    'account_user': new_account_user
                })

    def test_switch_account_owner(self):
        """Ensure that the account owner can be cleanly switched"""
        from accounts.models import AccountUser
        from accounts.utils import create_account, change_owner
        account = create_account(name="mytest", owner=self.user)
        secondary_user = AccountUser.objects.create(account=account,
                user=self.seconduser)
        new_owner = change_owner(account, secondary_user)
        self.assertEqual(new_owner.account_user, account.owner.account_user)

    def test_account_owner_is_member(self):
        """Ensure that the account owner must be a member of the group"""
        from accounts.models import AccountUser, AccountOwner
        from accounts.exceptions import AccountMismatch
        from accounts.utils import create_account

        account = create_account(name="mytest", owner=self.user)
        second_account = create_account(name="blah", owner=self.seconduser)
        second_account_user = AccountUser.objects.create(
                account=second_account, user=self.thirduser)

        first_owner = AccountOwner.objects.get(account=account)
        first_owner.account_user = second_account_user
        self.assertRaises(AccountMismatch, first_owner.save)

    def test_no_orphans(self):
        """Ensure that the account owner cannot be deleted"""
        from accounts.models import AccountUser
        from accounts.exceptions import OwnershipRequired
        from accounts.utils import create_account
        account = create_account(name="mytest", owner=self.user)
        secondary_user = AccountUser.objects.create(account=account,
                user=self.seconduser)
        owner_user = AccountUser.objects.get(account=account, user=self.user)
        self.assertRaises(OwnershipRequired, owner_user.delete)

    def test_account_user_delete(self):
        """Ensure special exceptions don't prohibit normal delete"""
        from accounts.models import AccountUser
        from accounts.utils import create_account
        account = create_account(name="mytest", owner=self.user)
        secondary_user = AccountUser.objects.create(account=account,
                user=self.seconduser)
        secondary_user.delete()

    def test_account_removal(self):
        """Ensure that deleting account removes account users"""
        from accounts.models import Account, AccountUser, AccountOwner
        from accounts.utils import create_account
        # Create an account
        account = create_account(name="mytest", owner=self.user)
        secondary_user = AccountUser.objects.create(account=account,
                user=self.seconduser)
        self.assertEqual(2, len(AccountUser.objects.filter(account=account)))

        account.delete()
        self.assertEqual(0, len(AccountUser.objects.filter(account=account)))





from django.test import TestCase
from django.core.urlresolvers import reverse


class BaseAccountURLsTest(TestCase):

    def test_account_list(self):
        reverse("base_account_list")

    def test_account_add(self):
        reverse("base_account_add")

    def test_account_detail(self):
        reverse("base_account_detail", kwargs={"account_pk": 1})

    def test_account_update(self):
        reverse("base_account_edit", kwargs={"account_pk": 1})

    def test_account_delete(self):
        reverse("base_account_delete", kwargs={"account_pk": 1})


class BaseAccountUserURLsTest(TestCase):

    def test_accountuser_list(self):
        reverse("base_accountuser_list", kwargs={"account_pk": 1})

    def test_accountuser_add(self):
        reverse("base_accountuser_add", kwargs={"account_pk": 1})

    def test_accountuser_detail(self):
        reverse("base_accountuser_detail", kwargs={"account_pk": 1,
            "accountuser_pk": 1})

    def test_accountuser_update(self):
        reverse("base_accountuser_edit", kwargs={"account_pk": 1,
            "accountuser_pk": 1})

    def test_accountuser_delete(self):
        reverse("base_accountuser_delete", kwargs={"account_pk": 1,
            "accountuser_pk": 1})

